@@ -71,7 +71,11 @@
 #include "debug.h"
 
 // really noisy debug
+#ifdef G_HAVE_ISO_VARARGS
+#define DD(...)
+#elif defined(G_HAVE_GNUC_VARARGS)
 #define DD(args...)
+#endif
 
 #include "totemPluginGlue.h"
 #include "totemPlugin.h"
@@ -575,8 +579,7 @@ totemPlugin::ViewerButtonPressed (guint aTimestamp, guint aButton)
 		}
 		if (g_ascii_strcasecmp (mTarget.get (), "myself") == 0 ||
 		    mTarget.Equals (NS_LITERAL_CSTRING ("_current")) ||
-		    mTarget.Equals (NS_LITERAL_CSTRING ("_self")) ||
-		    mTarget.Equals (NS_LITERAL_CSTRING (""))) {
+		    mTarget.Equals (NS_LITERAL_CSTRING ("_self"))) {
 			D ("Opening movie '%s'", mHref.get ());
 			dbus_g_proxy_call_no_reply (mViewerProxy,
 						    "SetHref",
