@@ -29,7 +29,6 @@
 #define HAVE_BACON_VIDEO_WIDGET_H
 
 #include <gtk/gtkbox.h>
-#include <popt.h>
 
 /* for optical disc enumeration type */
 #include "totem-disc.h"
@@ -43,9 +42,11 @@ G_BEGIN_DECLS
 #define BVW_ERROR bacon_video_widget_error_quark ()
 
 typedef struct BaconVideoWidgetPrivate BaconVideoWidgetPrivate;
+typedef struct BaconVideoWidgetCommon BaconVideoWidgetCommon;
 
 typedef struct {
 	GtkBox parent;
+	BaconVideoWidgetCommon *com;
 	BaconVideoWidgetPrivate *priv;
 } BaconVideoWidget;
 
@@ -96,7 +97,7 @@ typedef enum {
 
 GQuark bacon_video_widget_error_quark		 (void) G_GNUC_CONST;
 GType bacon_video_widget_get_type                (void);
-struct poptOption *bacon_video_widget_get_popt_table    (void);
+GOptionGroup* bacon_video_widget_get_option_group (void);
 /* This can be used if the app does not use popt */
 void bacon_video_widget_init_backend		 (int *argc, char ***argv);
 
@@ -149,6 +150,8 @@ int bacon_video_widget_get_volume                (BaconVideoWidget *bvw);
 /* Properties */
 void bacon_video_widget_set_logo		 (BaconVideoWidget *bvw,
 						  char *filename);
+void bacon_video_widget_set_logo_pixbuf		 (BaconVideoWidget *bvw,
+						  GdkPixbuf *logo);
 void  bacon_video_widget_set_logo_mode		 (BaconVideoWidget *bvw,
 						  gboolean logo_mode);
 gboolean bacon_video_widget_get_logo_mode	 (BaconVideoWidget *bvw);
@@ -198,7 +201,6 @@ typedef enum {
 	BVW_INFO_VIDEO_BITRATE,
 	BVW_INFO_VIDEO_CODEC,
 	BVW_INFO_FPS,
-	BVW_INFO_STILL_IMAGE,
 	/* Audio */
 	BVW_INFO_HAS_AUDIO,
 	BVW_INFO_AUDIO_BITRATE,
@@ -323,7 +325,7 @@ typedef enum {
 gboolean bacon_video_widget_fullscreen_mode_available (BaconVideoWidget *bvw,
 						       TvOutType tvout);
 
-gboolean bacon_video_widget_set_tv_out           (BaconVideoWidget *bvw,
+void bacon_video_widget_set_tv_out               (BaconVideoWidget *bvw,
 						  TvOutType tvout);
 TvOutType bacon_video_widget_get_tv_out          (BaconVideoWidget *bvw);
 
