@@ -36,6 +36,20 @@
 
 #include "debug.h"
 
+/* Helper function to escape underscores in labels
+ * before putting them in menu items */
+static char *
+escape_label_for_menu (const char *name)
+{
+	char *new, **a;
+
+	a = g_strsplit (name, "_", -1);
+	new = g_strjoinv ("__", a);
+	g_strfreev (a);
+
+	return new;
+}
+
 /* ISO-639 helpers */
 static GHashTable *lang_table;
 
@@ -227,29 +241,27 @@ add_lang_action (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 		int lang_id, GSList **group)
 {
 	const char *full_lang;
+	char *label;
 	char *name;
 	GtkAction *action;
 
 	full_lang = totem_lang_get_full (lang);
-
+	label = escape_label_for_menu (full_lang ? full_lang : lang);
 	name = g_strdup_printf ("%s-%d", prefix, lang_id);
 
 	action = g_object_new (GTK_TYPE_RADIO_ACTION,
-			"name", name,
-			"label", full_lang ? full_lang : lang,
-			"value", lang_id,
-			NULL);
-
-	gtk_radio_action_set_group (GTK_RADIO_ACTION (action),
-			*group);
+			       "name", name,
+			       "label", label,
+			       "value", lang_id,
+			       NULL);
+	g_free (label);
 
+	gtk_radio_action_set_group (GTK_RADIO_ACTION (action), *group);
 	*group = gtk_radio_action_get_group (GTK_RADIO_ACTION (action));
-
 	gtk_action_group_add_action (action_group, action);
 	g_object_unref (action);
-
 	gtk_ui_manager_add_ui (totem->ui_manager, ui_id,
-			path, name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
+			       path, name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
 	g_free (name);
 
 	return action;
@@ -261,9 +273,11 @@ create_lang_actions (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 		gboolean is_lang)
 {
 	GtkAction *action = NULL;
-	int i;
+	unsigned int i, *hash_value;
 	GList *l;
 	GSList *group = NULL;
+	GHashTable *lookup;
+	char *action_data;
 
 	if (is_lang == FALSE) {
 		add_lang_action (totem, action_group, ui_id, path, prefix,
@@ -274,14 +288,26 @@ create_lang_actions (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 			_("Auto"), -1, &group);
 
 	i = 0;
+	lookup = g_hash_table_new_full (g_str_hash, g_int_equal, g_free, NULL);
 
 	for (l = list; l != NULL; l = l->next)
 	{
+		hash_value = g_hash_table_lookup (lookup, l->data);
+		if (hash_value == NULL) {
+			action_data = g_strdup (l->data);
+			g_hash_table_insert (lookup, l->data, (unsigned int *)1);
+		} else {
+			action_data = g_strdup_printf ("%s #%u", (char *)l->data, (unsigned int)hash_value+1);
+			g_hash_table_replace (lookup, l->data, (unsigned int *)((unsigned int)hash_value+1));
+		}
+
 		add_lang_action (totem, action_group, ui_id, path, prefix,
-				l->data, i, &group);
-		i++;
+				 action_data, i, &group);
+ 		i++;
 	}
 
+	g_hash_table_destroy (lookup);
+
 	return action;
 }
 
@@ -475,6 +501,54 @@ totem_compare_recent_items (GtkRecentInfo *a, GtkRecentInfo *b)
 	return 0;
 }
 
+/* Copied from eel so we don't have a dependency on them.
+ * It's being consolidated into glib eventually anyway.
+ */
+static char *
+totem_str_middle_truncate (const char *string,
+			      guint truncate_length)
+{
+	char *truncated;
+	guint length;
+	guint num_left_chars;
+	guint num_right_chars;
+
+	const char delimter[] = "...";
+	const guint delimter_length = strlen (delimter);
+	const guint min_truncate_length = delimter_length + 2;
+
+	if (string == NULL) {
+		return NULL;
+	}
+
+	/* It doesnt make sense to truncate strings to less than
+	 * the size of the delimiter plus 2 characters (one on each
+	 * side)
+	 */
+	if (truncate_length < min_truncate_length) {
+		return g_strdup (string);
+	}
+
+	length = strlen (string);
+
+	/* Make sure the string is not already small enough. */
+	if (length <= truncate_length) {
+		return g_strdup (string);
+	}
+
+	/* Find the 'middle' where the truncation will occur. */
+	num_left_chars = (truncate_length - delimter_length) / 2;
+	num_right_chars = truncate_length - num_left_chars - delimter_length + 1;
+
+	truncated = g_new (char, truncate_length + 1);
+
+	strncpy (truncated, string, num_left_chars);
+	strncpy (truncated + num_left_chars, delimter, delimter_length);
+	strncpy (truncated + num_left_chars + delimter_length, string + length - num_right_chars + 1, num_right_chars);
+	
+	return truncated;
+}
+
 static void
 totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *totem)
 {
@@ -493,15 +567,18 @@ totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *
 	totem->recent_action_group = gtk_action_group_new ("recent-action-group");
 	gtk_ui_manager_insert_action_group (totem->ui_manager,
 			totem->recent_action_group, -1);
- 
+
 	items = gtk_recent_manager_get_items (recent_manager);
 	items = g_list_sort (items, (GCompareFunc) totem_compare_recent_items);
 
 	for (l = items; l && l->data; l = g_list_next (l)) {
 		GtkRecentInfo *info;
 		GtkAction     *action;
-		gchar         *action_name;
-		gchar         *label;
+		char          *action_name;
+		char          *label;
+		char          *escaped_label;
+		char          *label_trunc;
+		const char    *uri;
 
 		info = (GtkRecentInfo *) l->data;
 
@@ -509,29 +586,41 @@ totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *
 			continue;
 
 		action_name = g_strdup_printf ("recent-file%u", i++);
-		label = g_strdup_printf ("_%d.  %s", n_items + 1,
-				gtk_recent_info_get_display_name (info));
+		uri = gtk_recent_info_get_uri (info);
+
+		/* Munge the URI for display */
+		if (g_str_has_prefix (uri, "file:///") == FALSE) {
+			label_trunc = totem_str_middle_truncate (uri, TOTEM_MAX_RECENT_ITEM_LEN);
+		} else {
+			label_trunc = totem_str_middle_truncate
+				(gtk_recent_info_get_display_name (info),
+				 TOTEM_MAX_RECENT_ITEM_LEN);
+		}
+		escaped_label = escape_label_for_menu (label_trunc);
+		g_free (label_trunc);
+		label = g_strdup_printf ("_%d.  %s", n_items + 1, escaped_label);
+		g_free (escaped_label);
 
 		action = g_object_new (GTK_TYPE_ACTION,
-			       "name", action_name,
-			       "label", label,
-			       NULL);
+				       "name", action_name,
+				       "label", label,
+				       NULL);
 
 		g_object_set_data_full (G_OBJECT (action), "recent-info",
-				gtk_recent_info_ref (info),
-				(GDestroyNotify) gtk_recent_info_unref);
+					gtk_recent_info_ref (info),
+					(GDestroyNotify) gtk_recent_info_unref);
 		g_signal_connect (G_OBJECT (action), "activate",
-			  	G_CALLBACK (on_recent_file_item_activated),
-				totem);
+				  G_CALLBACK (on_recent_file_item_activated),
+				  totem);
 
 		gtk_action_group_add_action (totem->recent_action_group,
-				action);
+					     action);
 		g_object_unref (action);
 
 		gtk_ui_manager_add_ui (totem->ui_manager, totem->recent_ui_id,
-			       "/tmw-menubar/movie/recent-placeholder",
-			       label, action_name, GTK_UI_MANAGER_MENUITEM,
-			       FALSE);
+				       "/tmw-menubar/movie/recent-placeholder",
+				       label, action_name, GTK_UI_MANAGER_MENUITEM,
+				       FALSE);
 
 		g_free (action_name);
 		g_free (label);
@@ -564,26 +653,39 @@ void
 totem_action_add_recent (Totem *totem, const char *filename)
 {
 	GtkRecentData data;
-	char *groups[] = { "Totem", NULL };
-	char *display;
+	char *groups[] = { NULL, NULL };
 
-	if (strstr (filename, "file:///") == NULL)
-		return;
-
-	display = g_filename_from_uri (filename, NULL, NULL);
-	if (display) {
-		data.display_name = g_filename_display_basename (display);
-		g_free (display);
-	}
-	data.description = NULL;
 	data.mime_type = gnome_vfs_get_mime_type (filename);
+
+	if (strstr (filename, "file:///") == NULL) {
+		/* It's a URI/stream */
+		groups[0] = "TotemStreams";
+		data.display_name = NULL;
+	} else {
+		char *display;
+
+		/* Local files with no mime-type probably don't exist */
+		if (data.mime_type == NULL)
+			return;
+
+		/* It's a local file */
+		display = g_filename_from_uri (filename, NULL, NULL);
+		if (display) {
+			data.display_name = g_filename_display_basename (display);
+			g_free (display);
+		}
+		groups[0] = "Totem";
+	}
+
+	data.description = NULL;
 	data.app_name = g_strdup (g_get_application_name ());
 	data.app_exec = g_strjoin (" ", g_get_prgname (), "%u", NULL);
 	data.groups = groups;
 	gtk_recent_manager_add_full (totem->recent_manager,
 			filename, &data);
 
-	g_free (data.display_name);
+	if (data.display_name != NULL)
+		g_free (data.display_name);
 	g_free (data.mime_type);
 	g_free (data.app_name);
 	g_free (data.app_exec);
@@ -633,7 +735,8 @@ my_gnome_vfs_volume_get_mount_path (GnomeVFSVolume *volume)
 static void
 add_device_to_menu (GObject *device, guint position, Totem *totem)
 {
-	char *name, *icon_name, *device_path, *label, *activation_uri;
+	char *name, *escaped_name, *icon_name, *device_path;
+	char *label, *activation_uri;
 	GtkAction *action;
 	gboolean disabled = FALSE;
 
@@ -646,6 +749,13 @@ add_device_to_menu (GObject *device, guint position, Totem *totem)
 			disabled = TRUE;
 		}
 		g_free (activation_uri);
+	} else {
+		if (GNOME_IS_VFS_DRIVE (device)) {
+			device_path = gnome_vfs_drive_get_device_path
+				(GNOME_VFS_DRIVE (device));
+			disabled = !totem_cd_has_medium (device_path);
+			g_free (device_path);
+		}
 	}
 
 	name = fake_gnome_vfs_device_get_something (device,
@@ -657,9 +767,11 @@ add_device_to_menu (GObject *device, guint position, Totem *totem)
 		&my_gnome_vfs_volume_get_mount_path,
 		&gnome_vfs_drive_get_device_path);
 
-	g_strchomp (name);
-	label = g_strdup_printf (_("Play Disc '%s'"), name);
+	g_strstrip (name);
+	escaped_name = escape_label_for_menu (name);
 	g_free (name);
+	label = g_strdup_printf (_("Play Disc '%s'"), escaped_name);
+	g_free (escaped_name);
 
 	name = g_strdup_printf (_("device%d"), position);
 	action = gtk_action_new (name, label, NULL, NULL);
@@ -974,6 +1086,7 @@ about_action_callback (GtkAction *action, Totem *totem)
 		"Chee Bin Hoh <cbhoh@gnome.org>",
 		NULL
 	};
+	char *license = totem_interface_get_license ();
 
 #ifdef HAVE_GTK_ONLY
 	frontend_type = N_("GTK+");
@@ -996,10 +1109,13 @@ about_action_callback (GtkAction *action, Totem *totem)
 				     "artists", artists,
 				     "translator-credits", _("translator-credits"),
 				     "logo-icon-name", "totem",
+				     "license", license,
+				     "wrap-license", TRUE,
 				     NULL);
 
 	g_free (backend_version);
 	g_free (description);
+	g_free (license);
 }
 
 static void
@@ -1068,6 +1184,13 @@ aspect_ratio_changed_callback (GtkRadioAction *action, GtkRadioAction *current,
 	totem_action_set_aspect_ratio (totem, gtk_radio_action_get_current_value (current));
 }
 
+static void
+clear_playlist_action_callback (GtkAction *action, Totem *totem)
+{
+	totem_playlist_clear (totem->playlist);
+	totem_action_set_mrl (totem, NULL);
+}
+
 static const GtkActionEntry entries[] = {
 	{ "movie-menu", NULL, N_("_Movie") },
 	{ "open", GTK_STOCK_OPEN, N_("_Open..."), "<control>O", N_("Open a file"), G_CALLBACK (open_action_callback) },
@@ -1079,6 +1202,7 @@ static const GtkActionEntry entries[] = {
 
 	{ "edit-menu", NULL, N_("_Edit") },
 	{ "take-screenshot", "camera-photo", N_("Take _Screenshot..."), "<control>S", N_("Take a screenshot"), G_CALLBACK (take_screenshot_action_callback) },
+	{ "clear-playlist", NULL, N_("_Clear Playlist"), NULL, N_("Clear playlist"), G_CALLBACK (clear_playlist_action_callback) },
 	{ "preferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL, NULL, G_CALLBACK (preferences_action_callback) },
 
 	{ "view-menu", NULL, N_("_View") },
@@ -1212,6 +1336,13 @@ totem_ui_manager_setup (Totem *totem)
 	gtk_action_group_add_action (totem->main_action_group, action);
 	g_object_unref (action);
 
+	/* Hide help if we're using GTK+ only */
+#ifdef HAVE_GTK_ONLY
+	action = gtk_action_group_get_action
+		(totem->main_action_group, "contents");
+	gtk_action_set_visible (action, FALSE);
+#endif /* HAVE_GTK_ONLY */
+
 	totem->zoom_action_group = gtk_action_group_new ("zoom-action-group");
 	gtk_action_group_set_translation_domain (totem->zoom_action_group,
 			GETTEXT_PACKAGE);
